﻿using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections.Generic;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule;

namespace TimelineViewTesting
{

    #region VideoFrameTimeInterval class
    public class VideoFrameTimeInterval : DateTimeInterval
    {
        #region Constants

        private const long FramesPerSecond = 25;
        private const long FramesPerMinute = FramesPerSecond * 60;
        private const long FramesPerHour = FramesPerMinute * 60;
        private const string FrameFormat = "000";

        #endregion Constants

        #region Members
        private UltraTimelineView control = null;
        private DateTime zero = DateTime.Today;
        private uint currentFrame = 0;
        #endregion Members

        #region Constructor
        /// <summary>
        /// Creates a new instance
        /// </summary>
        public VideoFrameTimeInterval( UltraTimelineView control, DateTime zero )
        {
            this.control = control;
            this.control.ColumnHeaderInitializing += new ColumnHeaderInitializingHandler(this.OnColumnHeaderInitializing);
            this.zero = zero;
        }

        public static void TimeFromFrame( long totalFrames, out int hour, out int minute, out int second, out int frame )
        {
            hour = minute = second = frame = 0;

            //  Get the total number of hours, minutes, seconds, frames
            long total = totalFrames;

            hour = (int)(total / VideoFrameTimeInterval.FramesPerHour);
            total -= (hour * VideoFrameTimeInterval.FramesPerHour);

            minute = (int)(total / VideoFrameTimeInterval.FramesPerMinute);
            total -= (minute * VideoFrameTimeInterval.FramesPerMinute);

            second = (int)(total / VideoFrameTimeInterval.FramesPerSecond);
            total -= (second * VideoFrameTimeInterval.FramesPerSecond);

            //  Express frames as one-based
            frame = (int)(total + 1);
        }

        private void OnColumnHeaderInitializing(object sender, ColumnHeaderInitializingEventArgs e)
        {
            //  Get the number of minutes since 'zero' time
            DateTime time = e.DateTimeRange.StartDateTime;

            //  The current frame is determined by the number of minutes
            //  since 'zero' time.
            double totalMinutes = time.Subtract( this.zero ).TotalMinutes;
            this.currentFrame = totalMinutes >= 0d ? (uint)(totalMinutes) : 0;

            //  Get the total number of hours, minutes, seconds, frames
            long total = this.currentFrame;
            int hours = 0;
            int minutes = 0;
            int seconds = 0;
            int frames = 0;

            VideoFrameTimeInterval.TimeFromFrame( this.currentFrame, out hours, out minutes, out seconds, out frames );

            //  Format the header text to display the 'time' in the format
            //  "hh:mm:ss:fff", i.e., hours:minutes:seconds:frames. Use triple
            //  zero format for the frame rate to differentiate it from the units
            //  of time.
            string headerText =
                string.Format(
                "{0}:{1}:{2}:{3}",
                hours.ToString("00"),
                minutes.ToString("00"),
                seconds.ToString("00"),
                frames.ToString(VideoFrameTimeInterval.FrameFormat) );
            
            //  Set the Text property of the event args
            e.Text = headerText;
        }
        #endregion Constructor

        #region OnDispose
        /// <summary>
        /// Calles when this object is disposed of
        /// </summary>
        protected override void OnDispose()
        {
            base.OnDispose();
        }
        #endregion OnDispose

        #region DateTimeInterval overrides

        /// <summary>
        /// Returns whether this instance generates cycles of fixed duration.
        /// </summary>
        protected override bool HasFixedDuration
        {
            get { return true; }
        }

        /// <summary>
        /// Returns the average amount of time spanned by a cycle generated by this instance.
        /// </summary>
        public override TimeSpan NormalSpan
        {
            get { return TimeSpan.FromMinutes( 1 ); }
        }

        /// <summary>
        /// Returns a date format similar to the one we use so that the metrics
        /// logic will calculate the right size for the header text.
        /// </summary>
        protected override string GetDefaultHeaderTextFormat()
        {
            return "hh:mm:ss:sss";
        }

        /// <summary>
        /// Returns a range of 10 minutes (change this as desired)
        /// </summary>
        protected override TimeSpan HorizontalScrollRange
        {
            get
            {
                return TimeSpan.FromMinutes(10);
            }
        }

        /// <summary>
        /// Returns the next or previous date for this interval as relative
        /// to the specified <paramref name="date"/>
        /// </summary>
        /// <param name="date">The date for which the next/previous date is to be returned.</param>
        /// <param name="forward">True for next date, false for previous.</param>
        /// <returns>The DateTime of the beginning of the cycle.</returns>
        public override DateTime GetNextDate(DateTime date, bool forward)
        {
            double val = forward ? 1f : -1f;
            return date.AddMinutes(val);
        }

        public override string FormatDateRange(DateTime start, DateTime end)
        {
            long frame = (long)(start.Subtract( this.zero ).TotalMinutes);
            return string.Format( "frame {0}", frame );
        }

        #endregion DateTimeInterval overrides
    }
    #endregion VideoFrameTimeInterval class
}
